-- 65C02 Core Wrapper
-- 09.12.2009 kommt bis jetzt nur bis zur BASIC-Statuszeile. IRQ/NMI-Fehler?

-- All rights reserved
--
-- Redistribution and use in source and synthezised forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- Redistributions of source code must retain the above copyright notice,
-- this list of conditions and the following disclaimer.
--
-- Redistributions in synthesized form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
--
-- Neither the name of the author nor the names of other contributors may
-- be used to endorse or promote products derived from this software without
-- specific prior written permission.
--
-- THIS CODE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.
--
-- You are responsible for any legal issues arising from your use of this code.
--
-- The latest version of this file can be found at: www.fpgaarcade.com
--

library ieee;
  use ieee.std_logic_1164.all;
  use ieee.std_logic_arith.all;
  use ieee.std_logic_unsigned.all;
library unisim;
use unisim.vcomponents.all;


entity r65c02_wrap is
  port (
    ADDR         : out   std_logic_vector(15 downto 0);
    DATA_O       : out   std_logic_vector(7 downto 0);
    DATA_I       : in   std_logic_vector(7 downto 0);
    --
    RST          : in    std_logic;
    IRQ          : in    std_logic;
    NMI          : in    std_logic;
	WR_N         : out    std_logic;
	BUSENA_N     : out    std_logic;
--	SYNC     : out    std_logic;
    HALT         : in    std_logic;
    HALTED       : out    std_logic;
    RD_N         : out    std_logic;
--    CLK_OUT       : out    std_logic;
    SYSCLK       : in    std_logic
    );
end;


architecture RTL of r65c02_wrap is
	signal CLK_dcm               : std_logic;

	signal clk_count			: std_logic_vector(3 downto 0) := (others => '0');
	signal clk_fb				: std_logic;

  -- cpu
	signal wr_intern				: std_logic:='1';
	signal rd_intern				: std_logic:='1';
	signal c_irq_l				: std_logic:='1';
	signal c_nmi_l				: std_logic:='1';
	signal reset_l				: std_logic:='1';
	signal clk_i				: std_logic;
	signal sync_i				: std_logic;
	signal busclk_i				: std_logic;
	signal halt_del			: std_logic;
	signal addr_intern		: std_logic_vector(15 downto 0) := (others => '0');
	signal DATA_O_intern		: std_logic_vector(7 downto 0) := (others => '0');


	component core is
	port( 
		clk_clk_i   : in     std_logic;
		d_i         : in     std_logic_vector (7 downto 0);
		irq_n_i     : in     std_logic;
		nmi_n_i     : in     std_logic;
		rdy_i       : in     std_logic;
		rst_rst_n_i : in     std_logic;
		so_n_i      : in     std_logic;
		a_o         : out    std_logic_vector (15 downto 0);
		d_o         : out    std_logic_vector (7 downto 0);
		rd_o        : out    std_logic;
		sync_o      : out    std_logic;
		wr_o        : out    std_logic
	);
	end component;
	
	

begin
-- wr schn gewesen, luft aber mit CLKFX_MULTIPLY > 2 noch amok (RAM zu langsam)
-- kann auch keine CLKs mit t>41 ns erzeugen
--
--DCM_inst : DCM
--generic map (
--	CLKDV_DIVIDE => 2.0, --  Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5
--								--     7.0,7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
--	CLKFX_DIVIDE => 2,   --  Can be any integer from 1 to 32
--	CLKFX_MULTIPLY => 2, --  Can be any integer from 2 to 32
--	CLKIN_DIVIDE_BY_2 => FALSE, --  TRUE/FALSE to enable CLKIN divide by two feature
--	CLKIN_PERIOD => 20.0,          --  Specify period of input clock
--	CLKOUT_PHASE_SHIFT => "NONE", --  Specify phase shift of NONE, FIXED or VARIABLE
--	CLK_FEEDBACK => "1X",         --  Specify clock feedback of NONE, 1X or 2X
--	DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS", --  SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or
--														--     an integer from 0 to 15
--	DFS_FREQUENCY_MODE => "LOW",     --  HIGH or LOW frequency mode for frequency synthesis
--	DLL_FREQUENCY_MODE => "LOW",     --  HIGH or LOW frequency mode for DLL
--	DUTY_CYCLE_CORRECTION => TRUE, --  Duty cycle correction, TRUE or FALSE
--	FACTORY_JF => X"C080",          --  FACTORY JF Values
--	PHASE_SHIFT => 0,        --  Amount of fixed phase shift from -255 to 255
--	SIM_MODE => "SAFE", -- Simulation: "SAFE" vs "FAST", see "Synthesis and Simulation
--							  -- Design Guide" for details
--	STARTUP_WAIT => FALSE) --  Delay configuration DONE until DCM LOCK, TRUE/FALSE
--	
--
--port map (
--	CLK0 => clk_fb,     -- 0 degree DCM CLK ouptput
--	CLK180 => open, -- 180 degree DCM CLK output
--	CLK270 => open, -- 270 degree DCM CLK output
--	CLK2X => open,   -- 2X DCM CLK output
--	CLK2X180 => open, -- 2X, 180 degree DCM CLK out
--	CLK90 => open,   -- 90 degree DCM CLK output
--	CLKDV => open,   -- Divided DCM CLK out (CLKDV_DIVIDE)
--	CLKFX => CLK_dcm,   -- DCM CLK synthesis out (M/D)
--	CLKFX180 => open, -- 180 degree CLK synthesis out
--	LOCKED => open, -- DCM LOCK status output
--	PSDONE => open, -- Dynamic phase adjust done output
--	STATUS => open, -- 8-bit DCM status bits output
--	CLKFB => clk_fb,   -- DCM clock feedback
--	CLKIN => SYSCLK,   -- Clock input (from IBUFG, BUFG or DCM)
--	PSCLK => '0',   -- Dynamic phase adjust clock input
--	PSEN => '0',     -- Dynamic phase adjust enable input
--	PSINCDEC => '0', -- Dynamic phase adjust increment/decrement
--	RST => '0'        -- DCM asynchronous reset input
--);


-- CLK_dcm <= SYSCLK;	-- ohne DCM
ADDR <= addr_intern;
DATA_O <= DATA_O_intern;

busclk_i<= clk_count(1);

p_wait : process(clk_i,RST)  -- sync delay
	begin
		if RST='1' then
			halt_del<='0';
		elsif rising_edge(clk_i) then
-- HALT sync fr DMA-Zugriff
			halt_del <= HALT;
		end if;
	end process;

p_clk : process(SYSCLK,RST)  -- clock divider for slow IO $0600
	begin
		if RST='1' then
			clk_count<="0000";
		elsif rising_edge(SYSCLK) then
			clk_count <= clk_count + 1;
			clk_i<= '0';
			if clk_count(1 downto 0) = "01" then 
				clk_i<= '1';
			end if;
		end if;
--		if (addr_intern(15 downto 8)= x"06") then
--			clk_i <= clk_count(3);
--		else 
--			clk_i <= clk_count(1);	-- ohne DCM
--		end if;
	end process;

--p_del : process(SYSCLK,RST)  -- clock divider and delay
--	begin
--		if RST='1' then
--
--		elsif rising_edge(SYSCLK) then
--			if halt_del = '0' then	-- 
--				ADDR <= addr_intern;	-- um 1 SYSCLK verzgert fr SRAM Data Hold Time
--				DATA_O <= DATA_O_intern;
--			end if;
--		end if;
--	end process;


  cpu : core -- main cpu
      port map (
          rst_rst_n_i   => reset_l,
          clk_clk_i     => clk_i,
          rdy_i     => '1', -- not halt_i,
          irq_n_i   => c_irq_l,
          nmi_n_i   => c_nmi_l,
          so_n_i    => '1',
          wr_o   => wr_intern,
          rd_o   => rd_intern,
          sync_o    => sync_i,
          a_o  => addr_intern,
          d_i      => DATA_I,
          d_o      => DATA_O_intern
      );
		
	HALTED<= halt_del and HALT;
	c_irq_l <= not IRQ;
	c_nmi_l <= not NMI;
	WR_N <= (not wr_intern) when (halt_del='0') else '1';
	RD_N <= (not rd_intern) when (halt_del='0')  else '1'; -- RD_N <= (not rd_intern) or busclk_i  ...
	reset_l <= not RST;
	BUSENA_N <= busclk_i when halt_del='0' else '1';

end RTL;
